
def plot(self, label=None, fig=None):
    import matplotlib.pyplot as plt

    samples_list = self.get_samples_list()

    if fig is None:
        fig = plt.figure(figsize=(15, 5)).gca()

    if label is None:
        label = f'Signal {len(fig.get_lines()) + 1}'

    fig.set_xlabel('Time')
    fig.set_ylabel('Value')
    fig.set_title('Signal Plot')
    fig.grid(True)
        
    def plot_sample(s, dt, color=None):
        if color is None:
            l, = fig.plot(s.time, s.value, marker='o')
            color = l.get_color()
            l = fig.plot([s.time, s.time+dt], [s.value, s.value_at(s.time+dt)], color=color, linestyle='-')
        else:
            l = fig.plot(s.time, s.value, marker='o', color=color)
            l = fig.plot([s.time, s.time+dt], [s.value, s.value_at(s.time+dt)], color=color, linestyle='-')
        return color,l
    
    if len(samples_list) > 1:
        s = samples_list[0]
        sn = samples_list[1]
        dt = sn.time-s.time
            
        col,l = plot_sample(s, dt)
        
        
        for i in range(1, len(samples_list)-1):
            s = samples_list[i]
            sn = samples_list[i+1]
            dt = sn.time-s.time
            plot_sample(s,dt,color=col)
    else:
        sn = samples_list[0]
        col = None
    
    col, l = plot_sample(sn, self.end_time-sn.time, col)
    l[0].set_label(label)
    #fig.plot(sn.time, sn.value, marker='o', color=col)
    fig.legend()
    fig.figure.canvas.draw()

    return fig


import sys
sys.path.append('../build')
import stlrom

driver=stlrom.STLDriver()
s="signal x\nphi:=alw_[0, 5] x[t]>0"
driver.parse_string(s)
sig_names= driver.get_signals_names()

nb_sample=15
nb_signal=1

driver.add_sample([0, 1])
driver.add_sample([2, -1])
driver.add_sample([4, -3])

robs = driver.get_online_rob("phi")
assert(robs[0]==-3) #, "Robustness is wrong.")
assert(robs[1]==-10000)  # "Lower robustness is wrong.")
assert(robs[2]==-3)  # "Upper robustness is wrong.")





import sys
sys.path.append('../build')
import stlrom
import matplotlib.pyplot as plt
import numpy as np
import time

def test_rob_from(formula, t0):
    rob1 = driver.get_online_rob(formula, t0)
    # display both formula and robustness in one formated line
    print("Robustness of formula ", formula, " at time ", t0, " is ", rob1)
    return rob1

driver=stlrom.STLDriver()
s="""
signal x, y

workspace := (x[t] >= 0 and x[t] <= 5 and y[t] >= 0 and y[t] <= 5)
puddle := (x[t] >= 2 and x[t] <= 3 and y[t] <= 5)
carpet := (y[t] <= 1 and x[t] >= 4)
goal := (y[t] >= 4 and x[t] >= 4)

# safety := (not puddle) or (ev_[0,3] carpet) # test 1
# safety := (not puddle) or (workspace until_[0, 3] carpet) # test 2
 safety := (not puddle) or ((not goal) until_[0, 3] carpet) # test 3
# safety := puddle => ((not goal) until_[0,3] carpet) # test 4

phi := (workspace and safety) until_[0,10] goal
"""
driver.parse_string(s)

driver.add_sample([0,  2 , 1 ])
driver.add_sample([0.5, -1, 2])
robs = driver.get_online_rob("phi", 0)

print(robs)



import sys
sys.path.append('../build')
import stlrom

driver=stlrom.STLDriver()
s="signal x,y\nphi:=x[t]>0 => y[t]>2"
driver.parse_string(s)

sig_names= driver.get_signals_names()
print(sig_names)

driver.add_sample([0, 1, 1])
driver.add_sample([0.5, -1, 2])
print(driver.data)
driver.disp()

robs = driver.get_online_rob("phi")
print(robs)
assert(robs[0]==-1) #, "Robustness is wrong.")
assert(robs[1]==-1)  # "Lower robustness is wrong.")
assert(robs[2]==-1)  # "Upper robustness is wrong.")





import sys
sys.path.append('../build')
import stlrom
import matplotlib.pyplot as plt
import numpy as np
import time

def test_rob_from(formula, t0):
    rob1 = driver.get_online_rob(formula, t0)
    # display both formula and robustness in one formated line
    print("Robustness of formula ", formula, " at time ", t0, " is ", rob1)
    return rob1

driver=stlrom.STLDriver()
s="""signal x,y
mu1 := x[t]>0
mu2 := abs(y[t])>0
mmu_and := mu1 and mu2"""

driver.parse_string(s)

driver.add_sample([0, 0, 0])
driver.add_sample([1.5, -1.25, -2 ])
driver.add_sample([2., -2., 2. ])

test_rob_from("mmu_and", 2.)



import sys
sys.path.append('../build')
import stlrom

driver=stlrom.STLDriver()
s="signal x,y\nphi:=x[t]>0 or y[t]<0"
driver.parse_string(s)
sig_names= driver.get_signals_names()

driver.add_sample([0, 1, -3])
#driver.add_sample([0.5, -1])

robs = driver.get_online_rob("phi")
assert(robs[0]==3) #, "Robustness is wrong.")
assert(robs[1]==3)  # "Lower robustness is wrong.")
assert(robs[2]==3)  # "Upper robustness is wrong.")





import sys
sys.path.append('../build')
import stlrom
import numpy as np
import time
import gc
import argparse

def rand_sample(t, dim):
    val = np.random.rand(dim)
    return np.concatenate(([t], val)) 

def test_timing(driver,formula, num_sample):
    
    start_time =time.time()
    for i in range(0,num_sample):
        driver.add_sample(rand_sample(i,2))
        driver.get_online_rob(formula)
    
    # display both formula and robustness in one formated line
    end_time = time.time()
    execution_time = end_time - start_time
    print(f"Execution time for {num_sample} samples: {execution_time:.4f} seconds")
    
    return execution_time

num_trace = 1
num_samples = 10000
s= "stress.stl"


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('--num_trace', type=int, default=1, help='Number of traces')
    parser.add_argument('--num_samples', type=int, default=10000, help='Number of samples')

    args = parser.parse_args()

    num_trace = args.num_trace
    num_samples = args.num_samples

    for n in range(0, num_trace):
        driver = stlrom.STLDriver()
        driver.parse_file(s)
        test_timing(driver, "phi", num_samples)


import sys
sys.path.append('../build')
import stlrom

driver=stlrom.STLDriver()
s="signal x,y\nphi:= not(x[t]>3) => (x[t]>0 until_[0, 3] y[t]<0)"
driver.parse_string(s)
sig_names= driver.get_signals_names()

driver.add_sample([0,  2 , 1 ])
driver.add_sample([0.5, -1, 2])

robs = driver.get_online_rob("phi")
assert(robs[0]<0) #, "Robustness is wrong.")
assert(robs[1]<0)  # "Lower robustness is wrong.")
assert(robs[2]<0)  # "Upper robustness is wrong.")


import sys
sys.path.append('../build')
import stlrom

nb_sample=15
nb_signal=1

driver=stlrom.STLDriver()
s="signal x\nphi:=x[t]>2"
driver.parse_string(s)
sig_names= driver.get_signals_names()
signal=stlrom.rand_trace_data(nb_sample+2,nb_signal)
print(signal)

for i in range (nb_sample) :
	driver.add_sample(signal[i]);

robs = driver.get_online_rob("phi")
print(robs)
assert(robs[0]<=robs[1]) #, "Robustness is wrong.")
assert(robs[1]<= robs[2])  # "Lower robustness is wrong.")



import os
import sys
import importlib
import matplotlib.pyplot as plt


# Get the absolute path to the directory containing the local module
stlrom_path = os.path.abspath('../build')

# Insert the local module path at the beginning of sys.path
sys.path.insert(0, stlrom_path)

import stlrom

# Reload the module to ensure it is loaded from the new path
importlib.reload(stlrom)

import stlrom as sr
import numpy as np

def print_signal(signal):
    for s in signal.get_samples_list():
        s.print_sample()
    print('begin_time ', signal.begin_time)
    print('end_time    ',signal.end_time)

stl_parser = stlrom.STLDriver()
stl_parser.parse_string("signal x, y")       # declaring signals named 'x' and 'y'
stl_parser.parse_string("param p=0.2, tau=5") # declaring parameters named 'p' and 'q'
# Let's add some formulas
stl_parser.parse_string("mu_x := x[t] > 0 ") # a atomic proposition
stl_parser.parse_string("mu_y := y[t] < p ") # another atomic proposition
stl_parser.parse_string("mu := mu_x and mu_y") # a compound proposition
stl_parser.parse_string("phi := ev_[0, tau] mu") # a temporal proposition
stl_parser.disp()

phi = stl_parser.get_monitor("phi")


phi.add_sample([0., 0., 0.])  # time, x, y
phi.add_sample([0.5, 1.1, 2.2])
#phi.set_interpol('PREVIOUS')

#phi.set_eval_time(0.,1.)
phi.eval_rob() # should be -2.

print_signal(phi.get_signal())
print(phi.rob)



import sys
sys.path.append('../build')
import stlrom

nb_sample=15
nb_signal=1

driver=stlrom.STLDriver()
s="signal x\nphi:=x[t]<2"
driver.parse_string(s)
sig_names= driver.get_signals_names()

driver.add_sample([0, 1])
#driver.add_sample([0.5, -1])

robs = driver.get_online_rob("phi")
assert(robs[0]==1) #, "Robustness is wrong.")
assert(robs[1]==1)  # "Lower robustness is wrong.")
assert(robs[2]==1)  # "Upper robustness is wrong.")


import stlrom

# Zone search test

# Predicates:
# p(t): the entity in the search zone at time t, [0, 1]
# q(t): the UAV in the search zone at time t, [0, 1]

# Specification
# G[a, b](p(t) -> F[a, b]q(t))

# number of samples
#nb_samples = 20

# number of signals
#nb_signal = 2

# instantiate an STLDriver object
monitor = stlrom.STLDriver()

# declare signals, parameters (optional), and formulas
monitor.parse_string("signal p, q")
monitor.parse_string("param tau=20")

# declare specification
monitor.parse_string("mu := ev_[0, tau] q[t]")
monitor.parse_string("phi := alw_[0, tau] p[t] => mu")


sig_names= monitor.get_signals_names()
print("Signal Names are: ", sig_names)


#Samples of the trace from ZoneSearchTrace2.csv
monitor.add_sample([0, 1, 0])
monitor.add_sample([1, 1, 0])
monitor.add_sample([2, 0, 0])
monitor.add_sample([3, 0, 0])
monitor.add_sample([4, 0, 1])
monitor.add_sample([5, 0, 1])
monitor.add_sample([6, 1, 1])
monitor.add_sample([7, 0, 0])
monitor.add_sample([8, 1, 0])
monitor.add_sample([9, 0, 1])
monitor.add_sample([10, 0, 1])
monitor.add_sample([11, 0, 0])
monitor.add_sample([12, 1, 0])
monitor.add_sample([13, 0, 0])
monitor.add_sample([14, 1, 0])
monitor.add_sample([15, 0, 1])
monitor.add_sample([16, 1, 1])
monitor.add_sample([17, 1, 1])
monitor.add_sample([18, 0, 0])
monitor.add_sample([19, 0, 0])

print("Trace data: ", monitor.data)
#monitor.disp()

rob_phi = monitor.get_online_rob("phi")
print("rob_phi = ", rob_phi)



import sys
sys.path.append('../build')
import stlrom
driver=stlrom.STLDriver()
s="signal x\nphi:= alw_[0., 3.] (x[t]<5)"
driver.parse_string(s)

driver.add_sample([0, 1]);
driver.add_sample([.1, 2]);
robs = driver.get_online_rob("phi")
print("Robs:")
print(robs)
driver.add_sample([.2, 6]);
robs = driver.get_online_rob("phi")
print(robs)



import sys
sys.path.append('../build')
import stlrom

stl_driver =stlrom.STLDriver()

s="""
signal x, y    # signal names
mux := x[t]>0  # simple predicate 

param a=1, b=2, p = -3   
muy := a*y[t] + b > p   # operation and parameter in predicate

phi1 := alw_[0, 5] mux               # always (G works too) 
phi2 := ev_[3, 4] (mux) or phi1      # eventually (or F)
phi_until := phi1 until_[0, 1] phi2  # until 
"""

# parse the formulas
stl_driver.parse_string(s)  # or write the above in spec.stl, and use parse_file('spec.stl')

# add data as timed samples
stl_driver.add_sample([0,  2 , 1 ])  # must be of the format [t, x_val, y_val]
stl_driver.add_sample([0.5, -3, 2])  # i.e., contain signal value with same order as declared
stl_driver.add_sample([2.1, 10, 20])


# create monitor for phi 
phi1 = stl_driver.get_monitor("phi1")
print('Robustness of phi1 at time 0.: ', phi1.eval_rob())
print('Robustness of phi1 at time 1.: ', phi1.eval_rob(1.))




import sys
sys.path.append('../build')
import stlrom

def test_rob_from(formula, t0):
    rob1 = driver.get_online_rob(formula, t0)
    # display both formula and robustness in one formated line
    print("Robustness of formula ", formula, " at time ", t0, " is ", rob1)
    return rob1

driver=stlrom.STLDriver()
s="""signal x,y
param p1=3,p2=-2
mu1 := x[t]>p1
mu2 := abs(y[t])>p2
mmu_or := mu1 or mu2 
mmu_and := mu1 and mu2 
phi:= alw_[0,3] mmu_or"""

driver.parse_string(s)

driver.add_sample([0, 0, 0])
driver.add_sample([1.5, -1.25, -2 ])
driver.add_sample([2., -2., 2. ])

# request user a value for t_in and loop until t_in is -1
t_in = 1.2
test_rob_from("phi", t_in)
test_rob_from("mu1", t_in)
test_rob_from("mu2", t_in)
test_rob_from("mmu_or", t_in)
test_rob_from("mmu_and", t_in)